@@ -8,6 +8,9 @@
 extern "C" {
 #endif
 
+__isl_constructor
+__isl_give isl_union_set *isl_union_set_from_basic_set(
+	__isl_take isl_basic_set *bset);
 __isl_constructor
 __isl_give isl_union_set *isl_union_set_from_set(__isl_take isl_set *set);
 __isl_give isl_union_set *isl_union_set_empty(__isl_take isl_space *dim);
