@@ -8,7 +8,7 @@ struct isl_ctx {
 	int			 opt_allocated;
 	struct isl_options	*opt;
 	void			*user_opt;
-	struct isl_arg		*user_arg;
+	struct isl_args		*user_args;
 
 	isl_int			zero;
 	isl_int			one;
@@ -18,8 +18,9 @@ struct isl_ctx {
 	isl_int			normalize_gcd;
 
 	int			n_cached;
+	int			n_miss;
 	struct isl_blk		cache[ISL_BLK_CACHE_SIZE];
-	struct isl_hash_table	name_hash;
+	struct isl_hash_table	id_table;
 
 	enum isl_error		error;
 
