@@ -1,7 +1,7 @@
 /*
  * Copyright 2010      INRIA Saclay
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, INRIA Saclay - Ile-de-France,
  * Parc Club Orsay Universite, ZAC des vignes, 4 rue Jacques Monod,
@@ -12,7 +12,7 @@
 #define ISL_MORHP_H
 
 #include <stdio.h>
-#include <isl/dim.h>
+#include <isl/space.h>
 #include <isl/mat.h>
 #include <isl/set.h>
 
@@ -42,7 +42,7 @@ __isl_give isl_morph *isl_morph_copy(__isl_keep isl_morph *morph);
 __isl_give isl_morph *isl_morph_identity(__isl_keep isl_basic_set *bset);
 void isl_morph_free(__isl_take isl_morph *morph);
 
-__isl_give isl_dim *isl_morph_get_ran_dim(__isl_keep isl_morph *morph);
+__isl_give isl_space *isl_morph_get_ran_space(__isl_keep isl_morph *morph);
 unsigned isl_morph_dom_dim(__isl_keep isl_morph *morph, enum isl_dim_type type);
 unsigned isl_morph_ran_dim(__isl_keep isl_morph *morph, enum isl_dim_type type);
 
@@ -50,12 +50,15 @@ __isl_give isl_morph *isl_morph_remove_dom_dims(__isl_take isl_morph *morph,
 	enum isl_dim_type type, unsigned first, unsigned n);
 __isl_give isl_morph *isl_morph_remove_ran_dims(__isl_take isl_morph *morph,
 	enum isl_dim_type type, unsigned first, unsigned n);
+__isl_give isl_morph *isl_morph_dom_params(__isl_take isl_morph *morph);
+__isl_give isl_morph *isl_morph_ran_params(__isl_take isl_morph *morph);
 
 __isl_give isl_morph *isl_morph_compose(__isl_take isl_morph *morph1,
 	__isl_take isl_morph *morph2);
 __isl_give isl_morph *isl_morph_inverse(__isl_take isl_morph *morph);
 
-void isl_morph_dump(__isl_take isl_morph *morph, FILE *out);
+void isl_morph_print_internal(__isl_take isl_morph *morph, FILE *out);
+void isl_morph_dump(__isl_take isl_morph *morph);
 
 __isl_give isl_morph *isl_basic_set_variable_compression(
 	__isl_keep isl_basic_set *bset, enum isl_dim_type type);
