@@ -1,7 +1,7 @@
 /*
  * Copyright 2008-2009 Katholieke Universiteit Leuven
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, K.U.Leuven, Departement
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
@@ -11,7 +11,10 @@
 #include <string.h>
 #include <strings.h>
 #include <isl/ctx.h>
-#include <isl/stream.h>
+#include <isl_stream_private.h>
+#include <isl/map.h>
+#include <isl/aff.h>
+#include <isl_val_private.h>
 
 struct isl_keyword {
 	char			*name;
@@ -64,7 +67,7 @@ enum isl_token_type isl_stream_register_keyword(struct isl_stream *s,
 	return keyword->type;
 }
 
-static struct isl_token *isl_token_new(struct isl_ctx *ctx,
+struct isl_token *isl_token_new(isl_ctx *ctx,
 	int line, int col, unsigned on_new_line)
 {
 	struct isl_token *tok = isl_alloc_type(ctx, struct isl_token);
@@ -78,12 +81,49 @@ static struct isl_token *isl_token_new(struct isl_ctx *ctx,
 	return tok;
 }
 
+/* Return the type of "tok".
+ */
+int isl_token_get_type(struct isl_token *tok)
+{
+	return tok ? tok->type : ISL_TOKEN_ERROR;
+}
+
+/* Given a token of type ISL_TOKEN_VALUE, return the value it represents.
+ */
+__isl_give isl_val *isl_token_get_val(isl_ctx *ctx, struct isl_token *tok)
+{
+	if (!tok)
+		return NULL;
+	if (tok->type != ISL_TOKEN_VALUE)
+		isl_die(ctx, isl_error_invalid, "not a value token",
+			return NULL);
+
+	return isl_val_int_from_isl_int(ctx, tok->u.v);
+}
+
+/* Given a token of type ISL_TOKEN_STRING, return the string it represents.
+ */
+__isl_give char *isl_token_get_str(isl_ctx *ctx, struct isl_token *tok)
+{
+	if (!tok)
+		return NULL;
+	if (tok->type != ISL_TOKEN_STRING)
+		isl_die(ctx, isl_error_invalid, "not a string token",
+			return NULL);
+
+	return strdup(tok->u.s);
+}
+
 void isl_token_free(struct isl_token *tok)
 {
 	if (!tok)
 		return;
 	if (tok->type == ISL_TOKEN_VALUE)
 		isl_int_clear(tok->u.v);
+	else if (tok->type == ISL_TOKEN_MAP)
+		isl_map_free(tok->u.map);
+	else if (tok->type == ISL_TOKEN_AFF)
+		isl_pw_aff_free(tok->u.pwaff);
 	else
 		free(tok->u.s);
 	free(tok);
@@ -105,6 +145,20 @@ void isl_stream_error(struct isl_stream *s, struct isl_token *tok, char *msg)
 			fprintf(stderr, "got value '");
 			isl_int_print(stderr, tok->u.v, 0);
 			fprintf(stderr, "'\n");
+		} else if (tok->type == ISL_TOKEN_MAP) {
+			isl_printer *p;
+			fprintf(stderr, "got map '");
+			p = isl_printer_to_file(s->ctx, stderr);
+			p = isl_printer_print_map(p, tok->u.map);
+			isl_printer_free(p);
+			fprintf(stderr, "'\n");
+		} else if (tok->type == ISL_TOKEN_AFF) {
+			isl_printer *p;
+			fprintf(stderr, "got affine expression '");
+			p = isl_printer_to_file(s->ctx, stderr);
+			p = isl_printer_print_pw_aff(p, tok->u.pwaff);
+			isl_printer_free(p);
+			fprintf(stderr, "'\n");
 		} else if (tok->u.s)
 			fprintf(stderr, "got token '%s'\n", tok->u.s);
 		else
@@ -153,7 +207,10 @@ struct isl_stream* isl_stream_new_file(struct isl_ctx *ctx, FILE *file)
 
 struct isl_stream* isl_stream_new_str(struct isl_ctx *ctx, const char *str)
 {
-	struct isl_stream *s = isl_stream_new(ctx);
+	struct isl_stream *s;
+	if (!str)
+		return NULL;
+	s = isl_stream_new(ctx);
 	if (!s)
 		return NULL;
 	s->str = str;
@@ -242,6 +299,8 @@ static enum isl_token_type check_keywords(struct isl_stream *s)
 		return ISL_TOKEN_AND;
 	if (!strcasecmp(s->buffer, "or"))
 		return ISL_TOKEN_OR;
+	if (!strcasecmp(s->buffer, "implies"))
+		return ISL_TOKEN_IMPLIES;
 	if (!strcasecmp(s->buffer, "not"))
 		return ISL_TOKEN_NOT;
 	if (!strcasecmp(s->buffer, "infty"))
@@ -264,6 +323,12 @@ static enum isl_token_type check_keywords(struct isl_stream *s)
 		return ISL_TOKEN_CEILD;
 	if (!strcasecmp(s->buffer, "floord"))
 		return ISL_TOKEN_FLOORD;
+	if (!strcasecmp(s->buffer, "mod"))
+		return ISL_TOKEN_MOD;
+	if (!strcasecmp(s->buffer, "ceil"))
+		return ISL_TOKEN_CEIL;
+	if (!strcasecmp(s->buffer, "floor"))
+		return ISL_TOKEN_FLOOR;
 
 	if (!s->keywords)
 		return ISL_TOKEN_IDENT;
@@ -330,8 +395,8 @@ static struct isl_token *next_token(struct isl_stream *s, int same_line)
 	    c == '+' ||
 	    c == '*' ||
 	    c == '%' ||
+	    c == '?' ||
 	    c == '^' ||
-	    c == '=' ||
 	    c == '@' ||
 	    c == '$' ||
 	    c == ',' ||
@@ -368,6 +433,7 @@ static struct isl_token *next_token(struct isl_stream *s, int same_line)
 		}
 	}
 	if (c == '-' || isdigit(c)) {
+		int minus = c == '-';
 		tok = isl_token_new(s->ctx, line, col, old_line != line);
 		if (!tok)
 			return NULL;
@@ -382,6 +448,15 @@ static struct isl_token *next_token(struct isl_stream *s, int same_line)
 			isl_stream_ungetc(s, c);
 		isl_stream_push_char(s, '\0');
 		isl_int_read(tok->u.v, s->buffer);
+		if (minus && isl_int_is_zero(tok->u.v)) {
+			tok->col++;
+			tok->on_new_line = 0;
+			isl_stream_push_token(s, tok);
+			tok = isl_token_new(s->ctx, line, col, old_line != line);
+			if (!tok)
+				return NULL;
+			tok->type = (enum isl_token_type) '-';
+		}
 		return tok;
 	}
 	if (isalpha(c) || c == '_') {
@@ -423,6 +498,21 @@ static struct isl_token *next_token(struct isl_stream *s, int same_line)
 		tok->u.s = strdup(s->buffer);
 		return tok;
 	}
+	if (c == '=') {
+		int c;
+		tok = isl_token_new(s->ctx, line, col, old_line != line);
+		if (!tok)
+			return NULL;
+		if ((c = isl_stream_getc(s)) == '=') {
+			tok->u.s = strdup("==");
+			tok->type = ISL_TOKEN_EQ_EQ;
+			return tok;
+		}
+		if (c != -1)
+			isl_stream_ungetc(s, c);
+		tok->type = (enum isl_token_type) '=';
+		return tok;
+	}
 	if (c == ':') {
 		int c;
 		tok = isl_token_new(s->ctx, line, col, old_line != line);
@@ -542,8 +632,16 @@ static struct isl_token *next_token(struct isl_stream *s, int same_line)
 		tok = isl_token_new(s->ctx, line, col, old_line != line);
 		if (!tok)
 			return NULL;
-		tok->type = ISL_TOKEN_NOT;
-		tok->u.s = strdup("!");
+		if ((c = isl_stream_getc(s)) == '=') {
+			tok->u.s = strdup("!=");
+			tok->type = ISL_TOKEN_NE;
+			return tok;
+		} else {
+			tok->type = ISL_TOKEN_NOT;
+			tok->u.s = strdup("!");
+		}
+		if (c != -1)
+			isl_stream_ungetc(s, c);
 		return tok;
 	}
 
